@@ -61,6 +61,7 @@ int width  = 800;
 int height = 600;
 int big  = 0;
 int srnd = 0;
+int fastcut = 0;
 int use_mgl = 0;
 int verbose = 0;
 int quality  = MGL_DRAW_NORM;
@@ -74,10 +75,9 @@ void mgls_prepare3v(mglData *ex, mglData *ey, mglData *ez);
 void save(mglGraph *gr,const char *name,const char *suf);
 void test(mglGraph *gr)
 {
-//	gr->Box();	gr->WritePNG("1.png");	return;
-	
 	mglParse par;
-	par.Execute(gr,"colorbar;size -2;crange 0 1;value 90");
+	par.Execute(gr,"plot [0,-1,nan,0.1,0.5,1.1] 'o'");	//fplot 'x'");	//"box:fplot '2*sin(pi*x)' 'ko'");
+// 	par.Execute(gr,"colorbar;size -2;crange 0 1;value 90");
 // 	par.Execute(gr,"fplot 'x';legend 'рус':legend");
 //	par.Execute(gr,"fsurf 'x*y':rasterize:clf:fplot 'sin(x)' '2q':axis");
 //	par.Execute(gr,"xrange 0 1:new x 10 '3*x':echo x\nnew a 10 'value([0,1,-1],3*x)':echo a\nnew b 10 'spline([0,1,-1],3*x)':echo b");
@@ -186,6 +186,7 @@ static struct option longopts[] =
 	{ "list",	no_argument,	NULL,		'l' },
 	{ "mgl",	no_argument,	&use_mgl,	1 },
 	{ "srnd",	no_argument,	&srnd,		1 },
+	{ "fast",	no_argument,	&fastcut,	1 },
 
 	{ "png",	no_argument,	&type,		0 },
 	{ "eps",	no_argument,	&type,		1 },
@@ -398,6 +399,7 @@ int main(int argc,char **argv)
 		delete gr;	return 0;	}
 	else if(dotest==3)
 	{
+		gr->SetFastCut(fastcut);
 		int qual[7]={0,1,2,4,5,6,8};
 		size_t ll=strlen(mmgl_dat_prepare)+1;
 		mglParse par;
@@ -462,6 +464,7 @@ int main(int argc,char **argv)
 	gr->VertexColor(false);	gr->Compression(false);
 	if(name[0]==0)
 	{
+		gr->SetFastCut(fastcut);
 		while(s->name[0])	// all samples
 		{
 			if(!strcmp(s->name,"icon"))	{	s++;	continue;	}
@@ -486,6 +489,7 @@ int main(int argc,char **argv)
 	}
 	else	// manual sample
 	{
+		gr->SetFastCut(fastcut);
 		mglSample tst;	tst.name=name;
 		int i=0;
 		for(i=0;samp[i].name[0];i++);	// determine the number of samples
